########################################################################
#                                                                      #
#  R script for Introduction to Time Series, Mark Pickup               #
#                                                                      #  
#  Note: The results in the text were produced using Stata13.          #
#        The following R code produces results that are similar        #
#        but not exactley the same (in some instances).                #
#                                                                      #
########################################################################

# set working directory
setwd("C:/#####################")

# load packages
library(sandwich)
library(foreign)
library(aod)
library(nlWaldTest)
library(lmtest)
library(rugarch)
library(urca)
library(fGarch)

# load data
load("thermodata.Rda")
load("googledata.Rda")
load("germandata.Rda")
load("ARdata.Rda")
load("MAdata.Rda")
load("AR2data.Rda")
load("EPAdata.Rda")
load("ARdata.Rda")
load("MAdata.Rda")
load("Canada93to00.Rda")
load("KSI.Rda")
load("poll.Rda")
load("germany.ldata.Rda")
load("trustdata.Rda")
load("Odata.Rda")
load("legsuccdata.Rda")


# Chapter 1

## Table 1.3
summary(lm(caprsocial ~ caspsocial, data=thermodata))


# Chapter 2

##Figure 2.1 and 2.2
vote.ts<-ts(germandata[, 1], start=c(1982,10), freq=12)
plot(vote.ts, ylab = "Vote Intention for Government")
acf(germandata["vote"], lag.max=25, ylab = "Autocorrelations of Vote", ci.type = "ma")


##Google Example
mean(googledata[210:313,"beatles"])
mean(googledata[314:411,"beatles"])

mean(googledata[210:313,"gaga"])
mean(googledata[314:411,"gaga"])

var(googledata[210:313,"beatles"])
var(googledata[314:411,"beatles"])

var(googledata[210:313,"gaga"])
var(googledata[314:411,"gaga"])

beatles.ts<-ts(googledata[, 2], start=c(2004,1), freq=52)

b.ar<-ar(beatles.ts[210:313], order.max=1, method = "ols")
b.ar$ar

b.ar<-ar(beatles.ts[314:411], order.max=1, method = "ols")
b.ar$ar

gaga.ts<-ts(googledata[, 3], start=c(2004,1), freq=52)

g.ar<-ar(gaga.ts[210:313], order.max=1, method = "ols")
g.ar$ar

g.ar<-ar(gaga.ts[314:411], order.max=1, method = "ols")
g.ar$ar


##Table 2.1
summary(lm(caprsocial ~ caspsocial + cacounter, data=thermodata))


# Chapter 3

##Figure 3.3
caprsocial.ts<-ts(thermodata[16:32, 6], start=c(1987), freq=1)
plot(caprsocial.ts, type="l", ylab = "Public's Relative Preference")


##Table 3.3
summary(lm(caprsocial ~ caspsocial + cacounter + cacounterx, data=thermodata))

                                            
##Figure 3.4
ukprsocial.ts<-ts(thermodata[4:24, 12], start=c(1975), freq=1)
plot(ukprsocial.ts, type="l", ylab = "Public's Relative Preference")


##Table 3.4
summary(lm(ukprsocial ~ ukspsocial + ukcounter + ukcounterx, data=thermodata))


##Table 3.5
ukprModel<-lm(ukprsocial ~ ukcounter + ukcounterx, data=thermodata[7:24,])
summary(ukprModel)
thermodata$ukprdetrend<-resid(ukprModel)


##Table 3.6
ukspModel<-lm(ukspsocial ~ ukcounter + ukcounterx, data=thermodata[7:24,])
summary(ukspModel)
thermodata$ukspdetrend<-resid(ukspModel)


##Figure 3.5
ukprdetrend.ts<-ts(thermodata[7:24,21], start=c(1979), freq=1)
ukspdetrend.ts<-ts(thermodata[7:24,22], start=c(1979), freq=1)
plot(cbind(ukprdetrend.ts, ukspdetrend.ts), type="l", xlab = "Year")
twoway (tsline ukprdetrend) (tsline ukspdetrend, lpattern(dash)) in 7/24, xlabel(1976(4)1996) ytitle(Detrended Data) scheme(s1manual)


##Table 3.7
summary(lm(ukprdetrend ~ 0 + ukspdetrend, data=thermodata[7:24,]))


##Figure 3.6
KSIdata <- read.dta("KSI_12.dta")
lnKSI.ts<-ts(KSIdata[,8], start=c(1969), freq=12)
plot(lnKSI.ts, type="l", ylab = "ln(KSI)", xlab = "Year and Month")


##Table 3.8
KSIModel<-lm(lnKSI ~ t + m1 + m2 + m3 + m4 + m5 + m6 + m7 + m8 + m9 + m10 + m11 + SBlaw + lnpetrol, data=KSIdata)
summary(KSIModel)
wald.test(b = coef(KSIModel), Sigma = vcov(KSIModel), Terms = 3:13)


##Table 3.9
caspModel<-lm(caspsocial ~ cacounter + cacounterx, data=thermodata[17:32,])
summary(caspModel)
ca.detrend.data<-data.frame(resid(caspModel))

caspdetrend.ts<-ts(ca.detrend.data, start=c(1988), freq=1)
ca.sp.ar<-ar.ols(caspdetrend.ts, order.max=1, intercept=0)
ca.sp.ar$ar
ca.sp.ar$asy.se.coef

caprModel<-lm(caprsocial ~ caspsocial + cacounter + cacounterx, data=thermodata[17:32,])
Box.test(resid(caspModel), lag = 6, type = c("Ljung-Box"), fitdf = 0)


##Table 3.10
caprModel<-lm(caprsocial ~ caspsocial + cacounter + cacounterx, data=thermodata[17:32,])
bgtest(caprModel, order = 1)
bgtest(caprModel, order = 2)
bgtest(caprModel, order = 3)
bgtest(caprModel, order = 4)
bgtest(caprModel, order = 5)
bgtest(caprModel, order = 6)


##Breusch-Pagan test statistics 
ukprModel<-lm(ukprsocial ~ ukspsocial + ukcounter + ukcounterx, data=thermodata[7:24,])
bptest(ukprModel, studentize = FALSE)
shapiro.test(resid(ukprModel))


##Table 3.11
usprModel<-lm(usprsocial ~ usspsocial + uscounter + uscounterx, data=thermodata[1:33,])
summary(usprModel)
Box.test(resid(usprModel), lag = 14, type = c("Ljung-Box"), fitdf = 0)


##Table 3.12
x.data<-data.frame(thermodata$usspsocial[1:33], thermodata$uscounter[1:33], thermodata$uscounterx[1:33])
names(x.data)<-c("usspsocial", "uscounter", "uscounterx")
arima(thermodata$usprsocial[1:33], order = c(1, 0, 0), xreg = x.data, method="ML")
usprModel<-arima(thermodata$usprsocial[1:33], order = c(1, 0, 0), xreg = x.data, method="ML")
Box.test(resid(usprModel), lag = 14, type = c("Ljung-Box"), fitdf = 0)


##Table 3.13
usprModel<-lm(usprsocial ~ usspsocial + uscounter + uscounterx, data=thermodata[1:33,])
coeftest(usprModel, vcov = NeweyWest(usprModel, lag = 1, prewhite=FALSE, adjust=TRUE))


##Table 3.14
summary(lm(hocsuccess ~  t + I(t^2), data=legsuccdata))


##Table 3.15
legsuccModel<-lm(hocsuccess ~  govparty + govpop + minority + govpop:minority, data=legsuccdata)
summary(legsuccModel)

legsuccModel$coefficients[3] + legsuccModel$coefficients[5]
nlWaldtest(legsuccModel, "(a[3]+a[5])", 0)
Box.test(resid(legsuccModel), lag = 18, type = c("Ljung-Box"), fitdf = 0)
bptest(legsuccModel, studentize = FALSE)
shapiro.test(resid(legsuccModel))


##Figure 3.10
plot(resid(legsuccModel), type="p", xlab=("Parliamentary Session"))


# Chapter 4

##Table 4.1
usprsocial.ts<-ts(thermodata[2:33,19], start=c(1972), freq=1)
usspsocial.ts<-ts(thermodata[2:33,20], start=c(1972), freq=1)
l.usprsocial.ts<-ts(thermodata[1:32,19], start=c(1972), freq=1)
l.usspsocial.ts<-ts(thermodata[1:32,20], start=c(1972), freq=1)
uscounterx.ts<-ts(thermodata[2:33,15], start=c(1972), freq=1)


usprModel<-lm(usprsocial.ts ~ l.usprsocial.ts + usspsocial.ts + l.usspsocial.ts + uscounterx.ts) 
summary(usprModel)


##Q-test
Box.test(resid(usprModel), lag = 14, type = c("Ljung-Box"), fitdf = 0)


##Wald test
nlWaldtest(usprModel, "(a[3]+a[4])/(1-a[2])", 0)


##Table 4.2
vote.ts<-ts(germandata[2:52,1], start=c(1982,11), freq=12)
gdp.ts<-ts(germandata[2:52,3], start=c(1982,11), freq=1)
unemp.ts<-ts(germandata[2:52,2], start=c(1982,11), freq=1)
inf.ts<-ts(germandata[2:52,4], start=c(1982,11), freq=1)
l.vote.ts<-ts(germandata[1:51,1], start=c(1982,11), freq=1)
l.gdp.ts<-ts(germandata[1:51,3], start=c(1982,11), freq=1)
l.unemp.ts<-ts(germandata[1:51,2], start=c(1982,11), freq=1)
l.inf.ts<-ts(germandata[1:51,4], start=c(1982,11), freq=1)
t.ts<-ts(germandata[1:51,5], start=c(1982,11), freq=1)

germanModel<-lm(vote.ts ~ l.vote.ts + gdp.ts + l.gdp.ts + inf.ts + l.inf.ts+ unemp.ts + l.unemp.ts + t.ts) 
summary(germanModel)


##Q-test
Box.test(resid(germanModel), lag = 23, type = c("Ljung-Box"), fitdf = 0)


##Wald test
nlWaldtest(germanModel, "(a[5]+a[6])/(1-a[2])", 0)


##Table 4.3
usprsocial.ts<-ts(thermodata$usprsocial[2:33], start=c(1982,12), freq=12)
usspsocial.ts<-ts(thermodata$usspsocial[2:33], start=c(1982,12), freq=12)
uscounterx.ts<-ts(thermodata$uscounterx[2:33], start=c(1982,12), freq=12)
l.usprsocial.ts<-ts(thermodata$usprsocial[1:32], start=c(1982,12), freq=12)

usprModel<-lm(usprsocial.ts ~ l.usprsocial.ts + usspsocial.ts + uscounterx.ts)
summary(usprModel)


##Portmanteau (Q) statistic 
Box.test(resid(usprModel), lag = 14, type = c("Ljung-Box"), fitdf = 0)


##Shapiro-Wilk normality test
shapiro.test(resid(usprModel))


##Wald test
nlWaldtest(usprModel, "(a[3])/(1-a[2])", 0)


##Table 4.4
y.ts<-AR2data$y[2:39]
x.ts<-AR2data$x[2:39]
l.y.ts<-AR2data$y[1:38]

AR1Model<-lm(y.ts ~ l.y.ts + x.ts)
summary(AR1Model)


##Portmanteau (Q) statistic 
Box.test(resid(AR1Model), lag = 17, type = c("Ljung-Box"), fitdf = 0)

y.ts<-AR2data$y[3:39]
x.ts<-AR2data$x[3:39]
l.y.ts<-AR2data$y[2:38]
ll.y.ts<-AR2data$y[1:37]

AR2Model<-lm(y.ts ~ l.y.ts + ll.y.ts + x.ts)
summary(AR2Model)

##Portmanteau (Q) statistic
Box.test(resid(AR2Model), lag = 16, type = c("Ljung-Box"), fitdf = 0)


##Table 4.5
govapprov.ts<-ts(germany.ldata$govapprov[1:262], start=c(1977,2), freq=12)
SPDtrend.ts<-ts(germany.ldata$SPDtrend[1:262], start=c(1977,2), freq=12)
CDUtrend.ts<-ts(germany.ldata$CDUtrend[1:262], start=c(1977,2), freq=12)
cyc1.ts<-ts(germany.ldata$cyc1[1:262], start=c(1977,2), freq=12)
cyc2.ts<-ts(germany.ldata$cyc2[1:262], start=c(1977,2), freq=12)

germanyModel<-lm(govapprov.ts ~ SPDtrend.ts + CDUtrend.ts + cyc1.ts + cyc2.ts)
summary(germanyModel)


##Table 4.6
govapdtdt.ts<-ts(resid(germanyModel)[2:262], start=c(1977,3), freq=12)
l.govapdtdt.ts<-ts(resid(germanyModel)[1:261], start=c(1977,3), freq=12)
growth.ts<-ts(germany.ldata$growth[2:262], start=c(1977,3), freq=12)
l.growth.ts<-ts(germany.ldata$growth[1:261], start=c(1977,3), freq=12)
unemp.ts<-ts(germany.ldata$unemp[2:262], start=c(1977,3), freq=12)
l.unemp.ts<-ts(germany.ldata$unemp[1:261], start=c(1977,3), freq=12)
inf.ts<-ts(germany.ldata$inf[2:262], start=c(1977,3), freq=12)
l.inf.ts<-ts(germany.ldata$inf[1:261], start=c(1977,3), freq=12)

germanyModel2<-lm(govapdtdt.ts ~ l.govapdtdt.ts + growth.ts + unemp.ts + inf.ts + l.growth.ts + l.unemp.ts + l.inf.ts)
summary(germanyModel2)

##F tests
nlWaldtest(germanyModel2, "a[3]+a[6]", 0)
nlWaldtest(germanyModel2, "a[5]+a[8]", 0)
nlWaldtest(germanyModel2, "a[4]+a[7]", 0)
nlWaldtest(germanyModel2, "(a[5]+a[8])/(1-a[2])", 0)
nlWaldtest(germanyModel2, "(a[4]+a[7])/(1-a[2])", 0)


##Test for ARCH effects
Box.test((resid(germanyModel2)^2), lag = 1, type = c("Ljung-Box"), fitdf = 0)


##Table 4.7
x.data<-cbind(resid(germanyModel)[11:251], germany.ldata$growth[12:252], germany.ldata$unemp[12:252], germany.ldata$inf[12:252], germany.ldata$growth[11:251], germany.ldata$unemp[11:251], germany.ldata$inf[11:251]) 
y.data<-cbind(resid(germanyModel)[12:252])
spec<-ugarchspec(variance.model=list(model = "fGARCH", submodel = "GARCH", garchOrder = c(1,0)), mean.model = list(external.regressors = x.data, armaOrder = c(0, 0)))
arch<-ugarchfit(spec=spec, data=y.data)
arch


##Wald tests -- Warning nlWaldtest has not been tested for uGARCHfit yet and seems to produce incorrect results
inf.Waldtest<-nlWaldtest(arch, "(a[4]+a[7])/a[1]", 0)
pchisq(inf.Waldtest,1,lower.tail=FALSE) 
unemp.Waldtest<-nlWaldtest(arch, "(a[3]+a[6])/a[1]", 0)
pchisq(unemp.Waldtest,1,lower.tail=FALSE)


##Table 4.8
x.data<-data.frame(EPAdata$bdiffplus[2:60], EPAdata$bdiffminus[2:60], EPAdata$Whitman[1:59], EPAdata$WhitmanQuit[1:59], EPAdata$Holmstead[1:59], EPAdata$Leavitt[1:59], EPAdata$Johnson[1:59], EPAdata$Sansonetti[1:59], EPAdata$NSR[1:59], EPAdata$CAIR[1:59])
names(x.data)<-c("bdiffplus", "bdiffminus", "l.Whitman", "l.WhitmanQuit", "l.Holmstead", "l.Leavitt", "l.Johnson", "l.Sansonetti", "l.NSR", "l.CAIR")
arima(EPAdata$Enforcements[2:60], order = c(0, 0, 2), fixed=c(0, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA), xreg = x.data, method="ML", transform.pars = TRUE)
EPAModel<-arima(EPAdata$Enforcements[2:60], order = c(0, 0, 2), fixed=c(0, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA), xreg = x.data, method="ML", transform.pars = TRUE)

##Portmanteau (Q) statistic 
Box.test(resid(EPAModel), lag = 27, type = c("Ljung-Box"), fitdf = 0)


##Table 4.9
arima(EPAdata$Enforcements[2:60], order = c(0, 0, 0),  xreg = x.data, method="ML", transform.pars = TRUE)
EPAModel<-arima(EPAdata$Enforcements[2:60], order = c(0, 0, 0),  xreg = x.data, method="ML", transform.pars = TRUE)

##Portmanteau (Q) statistic 
Box.test(resid(EPAModel), lag = 27, type = c("Ljung-Box"), fitdf = 0)


# Chapter 5

##Figure 5.1
acf(ARdata["AR0_9"], lag.max=40, ylab = "Autocorrelation Function for Single Realisation", ci.type = "ma")
acf(ARdata["ARneg0_9"], lag.max=40, ylab = "Autocorrelation Function for Single Realisation", ci.type = "ma")


##Figure 5.2
pacf(ARdata["AR0_9"], lag.max=40, ylab = "Partial Autocorrelation Function for Single Realisation")
pacf(ARdata["ARneg0_9"], lag.max=40, ylab = "Partial Autocorrelation Function for Single Realisation")


##Figure 5.3
acf(MAdata$MApos[2:1000], lag.max=40, ylab = "Autocorrelation Function for Single Realisation", ci.type = "ma")
acf(MAdata$MAneg[2:1000], lag.max=40, ylab = "Autocorrelation Function for Single Realisation", ci.type = "ma")


##Figure 5.4
pacf(MAdata$MApos[2:1000], lag.max=40, ylab = "Partial Autocorrelation Function for Single Realisation")
pacf(MAdata$MAneg[2:1000], lag.max=40, ylab = "Partial Autocorrelation Function for Single Realisation")


##Figure 5.5
libvote.ts<-ts(Canada93to00$libvote[1:84], start=c(1993, 1), freq=12)
plot(libvote.ts, type="l", xlab = "Year", ylab = "Proportion Indicating Liberal Vote")


##Table 5.3
libtrend.ts<-ts(Canada93to00$libtrend[1:84], start=c(1993, 1), freq=12)
libvote.model.ts<-lm(libvote.ts ~ libtrend.ts)
summary(libvote.model.ts)


##Figure 5.6
libvote.dt<-ts(resid(libvote.model.ts), start=c(1993, 1), freq=12)
plot(libvote.dt, type="l", xlab = "Year", ylab = "Proportion Indicating Liberal Vote, Detrended")


##Figure 5.7
acf(libvote.dt, lag.max=30, ylab = "ACF for Detrended Vote", ci.type = "ma")
pacf(libvote.dt, lag.max=30, ylab = "PACF for Detrended Vote")


##Table 5.4
arima(libvote.dt, order = c(4, 0, 0), fixed=c(NA, 0, 0, NA, NA), method="ML", transform.pars = FALSE)
libvote.ar.1.4<-arima(libvote.dt, order = c(4, 0, 0), fixed=c(NA, 0, 0, NA, NA), method="ML", transform.pars = FALSE)

##Portmanteau (Q) statistic, and ACF and PACF
Box.test(resid(libvote.ar.1.4), lag = 40, type = c("Ljung-Box"), fitdf = 0)


##Figure 5.8,
acf(resid(libvote.ar.1.4), lag.max=30, ylab = "ACF for Residuals", ci.type = "ma")
pacf(resid(libvote.ar.1.4), lag.max=30, ylab = "PACF for Residuals")


##Table 5.5
arima(libvote.dt, order = c(7, 0, 0), fixed=c(NA, 0, 0, NA, 0, 0, NA, NA), method="ML", transform.pars = FALSE)


##Table 5.6
arima(libvote.dt, order = c(4, 0, 7), fixed=c(NA, 0, 0, NA, 0, 0, 0, 0, 0, 0, NA, NA), method="ML", transform.pars = FALSE)
libvote.ar.1.4.ma.7<-arima(libvote.dt, order = c(4, 0, 7), fixed=c(NA, 0, 0, NA, 0, 0, 0, 0, 0, 0, NA, NA), method="ML", transform.pars = FALSE)


##Portmanteau (Q) statistic, and ACF and PACF
Box.test(resid(libvote.ar.1.4.ma.7), lag = 40, type = c("Ljung-Box"))


##Figure 5.9
acf(resid(libvote.ar.1.4.ma.7), lag.max=30, ylab = "ACF for Residuals", ci.type = "ma")
pacf(resid(libvote.ar.1.4.ma.7), lag.max=30, ylab = "PACF for Residuals")


##Likelihood ratio test
libvote.ar.1.4.ma.7<-arima(libvote.dt, order = c(4, 0, 7), fixed=c(NA, 0, 0, NA, 0, 0, 0, 0, 0, 0, NA, NA), method="ML", transform.pars = FALSE)
libvote.ar.1.4<-arima(libvote.dt, order = c(4, 0, 0), fixed=c(NA, 0, 0, NA, NA), method="ML", transform.pars = FALSE)
lrtest(libvote.ar.1.4.ma.7, libvote.ar.1.4)


##Table 5.7
x.data<-data.frame(Canada93to00$gdp[4:81],  Canada93to00$inf[4:81], Canada93to00$unemp[1:78])
names(x.data)<-c("gdp", "inf", "unemp")
arima(libvote.dt[7:84], order = c(4, 0, 7), fixed=c(NA, 0, 0, NA, 0, 0, 0, 0, 0, 0, NA, NA, NA, NA, NA), xreg = x.data, method="ML", transform.pars = FALSE)


##Table 5.8
arima(libvote.dt[7:84], order = c(0, 0, 7), fixed=c( 0, 0, 0, 0, 0, 0, NA, NA, NA, NA, NA), xreg = x.data, method="ML", transform.pars = FALSE)


##Portmanteau (Q) statistic
libvote.ma.7<-arima(libvote.dt[7:84], order = c(0, 0, 7), fixed=c( 0, 0, 0, 0, 0, 0, NA, NA, NA, NA, NA), xreg = x.data, method="ML", transform.pars = FALSE)
Box.test(resid(libvote.ma.7), lag = 37, type = c("Ljung-Box"))


##Figure 5.10
acf(resid(libvote.ma.7), lag.max=30, ylab = "ACF for Residuals", ci.type = "ma")
pacf(resid(libvote.ma.7), lag.max=30, ylab = "PACF for Residuals")


##AIC comparison
arima(libvote.dt[7:84], order = c(0, 0, 7), fixed=c( 0, 0, 0, 0, 0, 0, NA, NA, NA, NA, NA), xreg = x.data, method="ML", transform.pars = FALSE)
arima(libvote.dt[7:84], order = c(4, 0, 7), fixed=c(NA, 0, 0, NA, 0, 0, 0, 0, 0, 0, NA, NA), method="ML", transform.pars = FALSE)


##Figure 5.11
x.data<-data.frame(KSI$m1,  KSI$m2, KSI$m3, KSI$m4, KSI$m5, KSI$m6, KSI$m7, KSI$m8, KSI$m9, KSI$m10, KSI$m11, KSI$lnpetrol)
KSI.model<-arima(KSI$lnKSI[1:169], order = c(0, 0, 0), xreg = x.data[1:169, 1:12], method="ML", transform.pars = FALSE)

acf(resid(KSI.model), lag.max=30, ylab = "ACF for Residuals", ci.type = "ma")
pacf(resid(KSI.model), lag.max=30, ylab = "PACF for Residuals")


##Table 5.9
arima(KSI$lnKSI[1:169], order = c(1, 0, 1), xreg = x.data[1:169, 1:12], method="ML", transform.pars = FALSE)


##Predicted values 
x.data<-data.frame(KSI$m1,  KSI$m2, KSI$m3, KSI$m4, KSI$m5, KSI$m6, KSI$m7, KSI$m8, KSI$m9, KSI$m10, KSI$m11, KSI$lnpetrol)
KSI.model<-arima(KSI$lnKSI[1:169], order = c(1, 0, 1), xreg = x.data[1:169, 1:12], method="ML", transform.pars = FALSE)
KSI.pred<-predict(KSI.model, newxreg=x.data)
plot(KSI.pred$pred)


##Figure 5.12
lnKSI.ts<-ts(KSI$lnKSI, start=c(1969, 1), freq=12)
KSI.pred.ts<-ts(KSI.pred$pred, start=c(1969, 1), freq=12)
ts.plot(cbind(KSI.pred.ts, lnKSI.ts), lty=1:2) 


##Table 5.10
x.data<-data.frame(KSI$m1,  KSI$m2, KSI$m3, KSI$m4, KSI$m5, KSI$m6, KSI$m7, KSI$m8, KSI$m9, KSI$m10, KSI$m11, KSI$lnpetrol, KSI$SBlaw)
arima(KSI$lnKSI, order = c(1, 0, 1), xreg = x.data, method="ML", transform.pars = FALSE)


##Portmanteau (Q) statistic
KSI.model<-arima(KSI$lnKSI, order = c(1, 0, 1), xreg = x.data, method="ML", transform.pars = FALSE)
Box.test(resid(KSI.model), lag = 40, type = c("Ljung-Box"), fitdf = 0)


##Table 5.11
x.data<-data.frame(KSI$m1,  KSI$m2, KSI$m3, KSI$m4, KSI$m5, KSI$m6, KSI$m7, KSI$m8, KSI$m9, KSI$m10, KSI$m11, KSI$lnpetrol, KSI$SBlaw_pulse)
arima(KSI$lnKSI, order = c(1, 0, 1), xreg = x.data, method="ML", transform.pars = FALSE)


##Table 5.12
B_Libtone.ts<-ts(poll$B_Libtone[2:51], start=1)
d.Liberror.ts<-ts(poll$d_Liberror[2:51], start=1)
acf(ts.union(B_Libtone.ts, d.Liberror.ts), na.action=na.pass, plot=FALSE, lag.max=5)


##Table 5.13
B_Libtone.ts<-ts(poll$B_Libtone[3:51], start=1)
d.Liberror.ts<-ts(poll$d_Liberror[3:51], start=1)
ld.Liberror.ts<-ts(poll$d_Liberror[2:50], start=1)
summary(lm(B_Libtone.ts ~ d.Liberror.ts + ld.Liberror.ts))


##Figure 5.13
govapprov.ts<-ts(germany.ldata$govapprov[1:262], start=c(1977,2), freq=12)
SPDtrend.ts<-ts(germany.ldata$SPDtrend[1:262], start=c(1977,2), freq=12)
CDUtrend.ts<-ts(germany.ldata$CDUtrend[1:262], start=c(1977,2), freq=12)
cyc1.ts<-ts(germany.ldata$cyc1[1:262], start=c(1977,2), freq=12)
cyc2.ts<-ts(germany.ldata$cyc2[1:262], start=c(1977,2), freq=12)

germanyModel<-lm(govapprov.ts ~ SPDtrend.ts + CDUtrend.ts + cyc1.ts + cyc2.ts)
summary(germanyModel)

govapdtdt.ts<-ts(resid(germanyModel)[2:262], start=c(1977,3), freq=12)
l.govapdtdt.ts<-ts(resid(germanyModel)[1:261], start=c(1977,3), freq=12)
growth.ts<-ts(germany.ldata$growth[2:262], start=c(1977,3), freq=12)
l.growth.ts<-ts(germany.ldata$growth[1:261], start=c(1977,3), freq=12)
unemp.ts<-ts(germany.ldata$unemp[2:262], start=c(1977,3), freq=12)
l.unemp.ts<-ts(germany.ldata$unemp[1:261], start=c(1977,3), freq=12)
inf.ts<-ts(germany.ldata$inf[2:262], start=c(1977,3), freq=12)
l.inf.ts<-ts(germany.ldata$inf[1:261], start=c(1977,3), freq=12)

germanyModel2<-lm(govapdtdt.ts ~ l.govapdtdt.ts + growth.ts + unemp.ts + inf.ts + l.growth.ts + l.unemp.ts + l.inf.ts)
summary(germanyModel2)

acf(resid(germanyModel2), lag.max=40, ylab = "ACF for Residuals", ci.type = "ma")
pacf(resid(germanyModel2), lag.max=40, ylab = "PACF for Residuals")


##Table 5.14a
x.data<-data.frame(l.govapdtdt.ts, growth.ts, unemp.ts, inf.ts, l.growth.ts, l.unemp.ts, l.inf.ts)
germanyModel3<-arima(govapdtdt.ts, order = c(0, 0, 1), xreg = x.data, method="ML", transform.pars = FALSE)


##Figure 5.14
acf(resid(germanyModel3)^2, na.action=na.pass, lag.max=40, ylab = "ACF for Squared Residuals", ci.type = "ma")
pacf(resid(germanyModel3)^2, na.action=na.pass, lag.max=40, ylab = "PACF for Squared Residuals")


##Table 5.14b
x.data<-cbind(resid(germanyModel)[11:251], germany.ldata$growth[12:252], germany.ldata$unemp[12:252], germany.ldata$inf[12:252], germany.ldata$growth[11:251], germany.ldata$unemp[11:251], germany.ldata$inf[11:251]) 
y.data<-cbind(resid(germanyModel)[12:252])
spec<-ugarchspec(variance.model=list(model = "fGARCH", submodel = "GARCH", garchOrder = c(1,1)), mean.model = list(external.regressors = x.data, armaOrder = c(0, 1)))
garch<-ugarchfit(spec=spec, data=y.data)
garch


##Portmanteau (Q) statistic, and ACF and PACF
Box.test(residuals(garch), lag = 40, type = c("Ljung-Box"))


##Figure 5.15
acf(residuals(garch), lag.max=40, ylab = "ACF of Residuals", ci.type = "ma")
pacf(residuals(garch), lag.max=40, ylab = "PACF of Residuals")


##Figure 5.16
acf(residuals(object = garch, standardize=TRUE), lag.max=40, ylab = "ACF of Standardized Residuals", ci.type = "ma")
pacf(residuals(object = garch, standardize=TRUE), lag.max=40, ylab = "PACF of Standardized Residuals")
Box.test(residuals(object = garch, standardize=TRUE), lag = 40, type = c("Ljung-Box"))


##LR test
govapprov.ts<-ts(germany.ldata$govapprov[1:262], start=c(1977,2), freq=12)
SPDtrend.ts<-ts(germany.ldata$SPDtrend[1:262], start=c(1977,2), freq=12)
CDUtrend.ts<-ts(germany.ldata$CDUtrend[1:262], start=c(1977,2), freq=12)
cyc1.ts<-ts(germany.ldata$cyc1[1:262], start=c(1977,2), freq=12)
cyc2.ts<-ts(germany.ldata$cyc2[1:262], start=c(1977,2), freq=12)

germanyModel<-lm(govapprov.ts ~ SPDtrend.ts + CDUtrend.ts + cyc1.ts + cyc2.ts)
summary(germanyModel)

x.data<-cbind(resid(germanyModel)[11:251], germany.ldata$growth[12:252], germany.ldata$unemp[12:252], germany.ldata$inf[12:252], germany.ldata$growth[11:251], germany.ldata$unemp[11:251], germany.ldata$inf[11:251]) 
y.data<-cbind(resid(germanyModel)[12:252])
spec<-ugarchspec(variance.model=list(model = "fGARCH", submodel = "GARCH", garchOrder = c(1,0)), mean.model = list(external.regressors = x.data, armaOrder = c(0, 0)))
arch<-ugarchfit(spec=spec, data=y.data)
arch

spec<-ugarchspec(variance.model=list(model = "fGARCH", submodel = "GARCH", garchOrder = c(1,1)), mean.model = list(external.regressors = x.data, armaOrder = c(0, 1)))
garch<-ugarchfit(spec=spec, data=y.data)
garch

l.ratio<--2*((likelihood(arch))-(likelihood(garch)))
l.ratio
pchisq(l.ratio,1,lower.tail=FALSE)


# Chapter 6

##Augmented Dickey-Fuller tests
trust.ts<-ts(trustdata$trust)

summary(ur.df(trust.ts, type= "drift", lags=0))
punitroot(-3.7859, trend = "c", statistic = "t")

summary(ur.df(trustdata$trust, type= "trend", lags=0))


##Table 6.1
summary(ur.df(trust.ts, type= "trend", lags=3))


##Phillips-Perron test
summary(ur.pp(trust.ts, model= "trend", lags="short", type = "Z-tau"))


##Figure 6.4
d.trust.ts<-ts(trustdata$trust[2:140]-trustdata$trust[1:139])
acf(d.trust.ts, na.action=na.pass, lag.max=40, ylab = "ACF for Differenced Trust", ci.type = "ma")
pacf(d.trust.ts, na.action=na.pass, lag.max=40, ylab = "PACF for Differenced Trust")


##Table 6.2
d.engage.ts<-ts(trustdata$engagement[2:140]-trustdata$engagement[1:139])
x.data<-d.engage.ts
trust.model<-arima(d.trust.ts, order = c(0, 0, 3), fixed=c( 0, 0, NA, NA, NA), xreg = x.data, method="ML", transform.pars = FALSE)
trust.model

##Figure 6.5
acf(resid(trust.model), na.action=na.pass, lag.max=30, ylab = "ACF for Residuals", ci.type = "ma")
pacf(resid(trust.model), na.action=na.pass, lag.max=30, ylab = "PACF for Residuals")


##Portmanteau (Q) statistic 
Box.test(resid(trust.model), lag = 40, type = c("Ljung-Box"), fitdf = 0)


##Figure 6.6, Ostrom.dta
us.de.ts<-ts(Odata$us_defence_expenditures, start=c(1967), freq=1)
ussr.de.ts<-ts(Odata$ussr_defence_expenditures, start=c(1967), freq=1)
def.exp.ts<-cbind(us.de.ts, ussr.de.ts)
ts.plot(def.exp.ts, gpars=list(xlab="Year", ylab="US Dollars (millions)", lty=c(1:2)))


##Augmented Dickey-Fuller tests
us.de.ts<-ts(Odata$us_defence_expenditures)
ussr.de.ts<-ts(Odata$ussr_defence_expenditures)
summary(ur.df(us.de.ts, type= "trend", lags=0))
summary(ur.df(us.de.ts, type= "drift", lags=0))
summary(ur.df(ussr.de.ts, type= "trend", lags=0))
summary(ur.df(ussr.de.ts, type= "drift", lags=0))


##Table 6.3
year.ts<-ts(Odata$year)
summary(lm(us.de.ts ~ ussr.de.ts + year.ts))


##Dickey-Fuller test
def.exp.model<-lm(us.de.ts ~ ussr.de.ts + year.ts)
summary(ur.df(resid(def.exp.model), type= "drift", lags=0))


##Table 6.4
d.us.de.ts<-ts(Odata$us_defence_expenditures[2:22] - Odata$us_defence_expenditures[1:21])
d.ussr.de.ts<-ts(Odata$ussr_defence_expenditures[2:22] - Odata$ussr_defence_expenditures[1:21])
l.residuals.ts<-resid(def.exp.model)[1:21]
summary(lm( d.us.de.ts ~ d.ussr.de.ts + l.residuals.ts - 1))


##Portmanteau (Q) statistic
def.exp.model2<-lm(d.us.de.ts ~ d.ussr.de.ts + l.residuals.ts - 1)
Box.test(resid(def.exp.model2), lag = 8, type = c("Ljung-Box"), fitdf = 0)


##Table 6.5
ld.ussr.de.ts<-ts(d.us.de.ts)
summary(lm(us.de.ts[3:22] ~ ussr.de.ts[3:22] + d.ussr.de.ts[2:21] + d.ussr.de.ts[1:20] + d.ussr.de.ts[3:22]+ year.ts[3:22]))


##Table 6.6
coint.data<-cbind(us.de.ts, ussr.de.ts)
summary(ca.jo(coint.data, type = c("eigen"), ecdet = c("trend"), K = 2, spec=c("longrun")))


##Table 6.7
d.vote<-ts(germandata$vote[2:52] - germandata$vote[1:51])
d.gdp<-ts(germandata$gdp[2:52] - germandata$gdp[1:51])
d.inf<-ts(germandata$inf[2:52] - germandata$inf[1:51])
d.unemp<-ts(germandata$unemp[2:52] - germandata$unemp[1:51])
summary(german.ecm<-lm(d.vote ~ germandata$vote[1:51] + d.gdp + germandata$gdp[1:51] + d.inf + germandata$inf[1:51] + d.unemp + germandata$unemp[1:51] + germandata$t[1:51]))


##F tests
nlWaldtest(german.ecm, "a[4]/a[2]", 0)
nlWaldtest(german.ecm, "a[6]/a[2]", 0)
nlWaldtest(german.ecm, "a[8]/a[2]", 0)


##Portmanteau (Q) statistic
Box.test(resid(german.ecm), lag = 23, type = c("Ljung-Box"), fitdf = 0)


##Breusch-Pagan test 
bptest(german.ecm, studentize = FALSE)
